#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['django_template_breakpoint', 'django_template_breakpoint.templatetags']

package_data = \
{'': ['*']}

install_requires = \
['django']

extras_require = \
{'test': ['pytest']}

setup(name='django_template_breakpoint',
      version='0.0.1',
      description='A template tag for dropping into a debugger inside a Django template.',
      author=None,
      author_email='Peter Inglesby <peter.inglesby@gmail.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.7',
     )



import inspect
import sys

import django
from django.conf import settings
from django.template import Context, Template


def test_breakpoint():
    # Set up Django.
    settings.configure(
        INSTALLED_APPS=["django_template_breakpoint"],
        TEMPLATES=[{"BACKEND": "django.template.backends.django.DjangoTemplates"}],
    )
    django.setup()

    # Set breakpoint hook to a function that verifies that the template variables have
    # been added to locals() as expected.
    sys.breakpointhook = verify_parent_locals

    # Build and render template.  We include a context key called "context" to ensure
    # that this special value is handled correctly.
    template_string = """
    {% load breakpoint %}

    {{ context.0 }}
    {% for item in items %}
    {% breakpoint %}
    {% endfor %}
    {{ context.1 }}
    """
    template = Template(template_string)
    context = Context({"items": ["XXX"], "context": ["Hello", "World"]})
    rendered = template.render(context)

    # Check that tag does not contribute any output to the rendered template.
    assert rendered.strip().split() == ["Hello", "World"]


def verify_parent_locals():
    frame = inspect.currentframe()
    f_locals = frame.f_back.f_locals

    assert f_locals["context"] == ["Hello", "World"]
    assert f_locals["items"] == ["XXX"]
    assert f_locals["item"] == "XXX"
    assert f_locals["forloop"]["counter0"] == 0



import builtins
import sys

from django import template

register = template.Library()


@register.simple_tag(takes_context=True)
def breakpoint(context):  # noqa: A001
    """Invoke the breakpoint hook.

    Since this function's parameter must be called `context`, we call another function
    to do the work

    This calls a separate function because this function is decorated with
    `takes_context = True`, which means that its first parameter must be called
    `context`.  However...
    """

    _breakpoint(context)

    # Return empty string, so that nothing is rendered in the template.
    return ""


def _breakpoint(_context):
    """Actually do the work."""

    # Add each template variable in the context to locals().  Inside the body of a
    # function, existing keys in locals() cannot be overwritten, but new keys can be
    # added.
    #
    # Because (a) all variables in this function begin with an underscore, and (b)
    # template variables cannot begin with an underscore, there is no danger that a
    # template variable will not be added to locals()
    for _k, _v in _context.flatten().items():
        locals()[_k] = _v

    # Invoke the breakpoint hook.  By default, this calls pdb.set_trace().
    builtins.breakpoint()
